@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2024 suzukimain
+# Copyright 2025 suzukimain
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,15 +17,11 @@ import os
 import re
 import types
 from collections import OrderedDict
-from dataclasses import asdict, dataclass
+from dataclasses import asdict, dataclass, field
 from typing import Dict, List, Optional, Union
 
 import requests
 import torch
-from huggingface_hub import hf_api, hf_hub_download
-from huggingface_hub.file_download import http_get
-from huggingface_hub.utils import validate_hf_hub_args
-
 from diffusers.loaders.single_file_utils import (
     VALID_URL_PREFIXES,
     _extract_repo_id_and_weights_name,
@@ -53,14 +49,19 @@ from diffusers.pipelines.stable_diffusion import (
     StableDiffusionPipeline,
     StableDiffusionUpscalePipeline,
 )
-from diffusers.pipelines.stable_diffusion_3 import StableDiffusion3Img2ImgPipeline, StableDiffusion3Pipeline
+from diffusers.pipelines.stable_diffusion_3 import (
+    StableDiffusion3Img2ImgPipeline,
+    StableDiffusion3Pipeline,
+)
 from diffusers.pipelines.stable_diffusion_xl import (
     StableDiffusionXLImg2ImgPipeline,
     StableDiffusionXLInpaintPipeline,
     StableDiffusionXLPipeline,
 )
 from diffusers.utils import logging
-
+from huggingface_hub import hf_api, hf_hub_download
+from huggingface_hub.file_download import http_get
+from huggingface_hub.utils import validate_hf_hub_args
 
 logger = logging.get_logger(__name__)
 
@@ -321,13 +322,15 @@ class SearchResult:
     model_path: str = ""
     loading_method: Union[str, None] = None
     checkpoint_format: Union[str, None] = None
-    repo_status: RepoStatus = RepoStatus()
-    model_status: ModelStatus = ModelStatus()
-    extra_status: ExtraStatus = ExtraStatus()
+    repo_status: RepoStatus = field(default_factory=RepoStatus)
+    model_status: ModelStatus = field(default_factory=ModelStatus)
+    extra_status: ExtraStatus = field(default_factory=ExtraStatus)
 
 
 @validate_hf_hub_args
-def load_pipeline_from_single_file(pretrained_model_or_path, pipeline_mapping, **kwargs):
+def load_pipeline_from_single_file(
+    pretrained_model_or_path, pipeline_mapping, **kwargs
+):
     r"""
     Instantiate a [`DiffusionPipeline`] from pretrained pipeline weights saved in the `.ckpt` or `.safetensors`
     format. The pipeline is set in evaluation mode (`model.eval()`) by default.
@@ -587,18 +590,28 @@ def search_huggingface(search_word: str, **kwargs) -> Union[str, SearchResult, N
     pipeline_tag = kwargs.pop("pipeline_tag", None)
     token = kwargs.pop("token", None)
     gated = kwargs.pop("gated", False)
+    cache_dir = kwargs.pop("cache_dir", None)
     skip_error = kwargs.pop("skip_error", False)
 
+    file_list = []
+    hf_repo_info = {}
+    hf_security_info = {}
+    model_path = ""
+    repo_id, file_name = "", ""
+    diffusers_model_exists = False
+
     # Get the type and loading method for the keyword
     search_word_status = get_keyword_types(search_word)
 
     if search_word_status["type"]["hf_repo"]:
+        hf_repo_info = hf_api.model_info(repo_id=search_word, securityStatus=True)
         if download:
             model_path = DiffusionPipeline.download(
                 search_word,
                 revision=revision,
                 token=token,
                 force_download=force_download,
+                cache_dir=cache_dir,
                 **kwargs,
             )
         else:
@@ -609,8 +622,9 @@ def search_huggingface(search_word: str, **kwargs) -> Union[str, SearchResult, N
             model_path = hf_hub_download(
                 repo_id=repo_id,
                 filename=weights_name,
-                force_download=force_download,
                 token=token,
+                force_download=force_download,
+                cache_dir=cache_dir,
             )
         else:
             model_path = search_word
@@ -620,7 +634,9 @@ def search_huggingface(search_word: str, **kwargs) -> Union[str, SearchResult, N
         if skip_error:
             return None
         else:
-            raise ValueError("The URL for Civitai is invalid with `for_hf`. Please use `for_civitai` instead.")
+            raise ValueError(
+                "The URL for Civitai is invalid with `for_hf`. Please use `for_civitai` instead."
+            )
     else:
         # Get model data from HF API
         hf_models = hf_api.list_models(
@@ -635,13 +651,6 @@ def search_huggingface(search_word: str, **kwargs) -> Union[str, SearchResult, N
         )
         model_dicts = [asdict(value) for value in list(hf_models)]
 
-        file_list = []
-        hf_repo_info = {}
-        hf_security_info = {}
-        model_path = ""
-        repo_id, file_name = "", ""
-        diffusers_model_exists = False
-
         # Loop through models to find a suitable candidate
         for repo_info in model_dicts:
             repo_id = repo_info["id"]
@@ -666,7 +675,8 @@ def search_huggingface(search_word: str, **kwargs) -> Union[str, SearchResult, N
                         any(file_path.endswith(ext) for ext in EXTENSION)
                         and not any(config in file_path for config in CONFIG_FILE_LIST)
                         and not any(exc in file_path for exc in exclusion)
-                        and os.path.basename(os.path.dirname(file_path)) not in DIFFUSERS_CONFIG_DIR
+                        and os.path.basename(os.path.dirname(file_path))
+                        not in DIFFUSERS_CONFIG_DIR
                     ):
                         file_list.append(file_path)
 
@@ -678,13 +688,16 @@ def search_huggingface(search_word: str, **kwargs) -> Union[str, SearchResult, N
             if skip_error:
                 return None
             else:
-                raise ValueError("No models matching your criteria were found on huggingface.")
+                raise ValueError(
+                    "No models matching your criteria were found on huggingface."
+                )
 
         if diffusers_model_exists:
             if download:
                 model_path = DiffusionPipeline.download(
                     repo_id,
                     token=token,
+                    cache_dir=cache_dir,
                     **kwargs,
                 )
             else:
@@ -693,7 +706,11 @@ def search_huggingface(search_word: str, **kwargs) -> Union[str, SearchResult, N
         elif file_list:
             # Sort and find the safest model
             file_name = next(
-                (model for model in sorted(file_list, reverse=True) if re.search(r"(?i)[-_](safe|sfw)", model)),
+                (
+                    model
+                    for model in sorted(file_list, reverse=True)
+                    if re.search(r"(?i)[-_](safe|sfw)", model)
+                ),
                 file_list[0],
             )
 
@@ -704,8 +721,13 @@ def search_huggingface(search_word: str, **kwargs) -> Union[str, SearchResult, N
                     revision=revision,
                     token=token,
                     force_download=force_download,
+                    cache_dir=cache_dir,
                 )
 
+    # `pathlib.PosixPath` may be returned
+    if model_path:
+        model_path = str(model_path)
+
     if file_name:
         download_url = f"https://huggingface.co/{repo_id}/blob/main/{file_name}"
     else:
@@ -718,7 +740,9 @@ def search_huggingface(search_word: str, **kwargs) -> Union[str, SearchResult, N
             model_path=model_path or download_url,
             loading_method=output_info["loading_method"],
             checkpoint_format=output_info["checkpoint_format"],
-            repo_status=RepoStatus(repo_id=repo_id, repo_hash=hf_repo_info.sha, version=revision),
+            repo_status=RepoStatus(
+                repo_id=repo_id, repo_hash=hf_repo_info.sha, version=revision
+            ),
             model_status=ModelStatus(
                 search_word=search_word,
                 site_url=download_url,
@@ -730,7 +754,7 @@ def search_huggingface(search_word: str, **kwargs) -> Union[str, SearchResult, N
         )
 
     else:
-        return model_path
+        return model_path or download_url
 
 
 def search_civitai(search_word: str, **kwargs) -> Union[str, SearchResult, None]:
@@ -809,7 +833,9 @@ def search_civitai(search_word: str, **kwargs) -> Union[str, SearchResult, None]
 
     try:
         # Make the request to the CivitAI API
-        response = requests.get("https://civitai.com/api/v1/models", params=params, headers=headers)
+        response = requests.get(
+            "https://civitai.com/api/v1/models", params=params, headers=headers
+        )
         response.raise_for_status()
     except requests.exceptions.HTTPError as err:
         raise requests.HTTPError(f"Could not get elements from the URL: {err}")
@@ -823,7 +849,9 @@ def search_civitai(search_word: str, **kwargs) -> Union[str, SearchResult, None]
                 raise ValueError("Invalid JSON response")
 
     # Sort repositories by download count in descending order
-    sorted_repos = sorted(data["items"], key=lambda x: x["stats"]["downloadCount"], reverse=True)
+    sorted_repos = sorted(
+        data["items"], key=lambda x: x["stats"]["downloadCount"], reverse=True
+    )
 
     for selected_repo in sorted_repos:
         repo_name = selected_repo["name"]
@@ -848,7 +876,8 @@ def search_civitai(search_word: str, **kwargs) -> Union[str, SearchResult, None]
                         model_data["pickleScanResult"] == "Success"
                         and model_data["virusScanResult"] == "Success"
                         and any(file_name.endswith(ext) for ext in EXTENSION)
-                        and os.path.basename(os.path.dirname(file_name)) not in DIFFUSERS_CONFIG_DIR
+                        and os.path.basename(os.path.dirname(file_name))
+                        not in DIFFUSERS_CONFIG_DIR
                     ):
                         file_status = {
                             "filename": file_name,
@@ -858,12 +887,16 @@ def search_civitai(search_word: str, **kwargs) -> Union[str, SearchResult, None]
 
             if models_list:
                 # Sort the models list by filename and find the safest model
-                sorted_models = sorted(models_list, key=lambda x: x["filename"], reverse=True)
+                sorted_models = sorted(
+                    models_list, key=lambda x: x["filename"], reverse=True
+                )
                 selected_model = next(
                     (
                         model_data
                         for model_data in sorted_models
-                        if bool(re.search(r"(?i)[-_](safe|sfw)", model_data["filename"]))
+                        if bool(
+                            re.search(r"(?i)[-_](safe|sfw)", model_data["filename"])
+                        )
                     ),
                     sorted_models[0],
                 )
@@ -878,7 +911,9 @@ def search_civitai(search_word: str, **kwargs) -> Union[str, SearchResult, None]
         if skip_error:
             return None
         else:
-            raise ValueError("No model found. Please try changing the word you are searching for.")
+            raise ValueError(
+                "No model found. Please try changing the word you are searching for."
+            )
 
     # Define model file status
     file_name = selected_model["filename"]
@@ -887,7 +922,9 @@ def search_civitai(search_word: str, **kwargs) -> Union[str, SearchResult, None]
     # Handle file download and setting model information
     if download:
         # The path where the model is to be saved.
-        model_path = os.path.join(str(civitai_cache_dir), str(repo_id), str(version_id), str(file_name))
+        model_path = os.path.join(
+            str(civitai_cache_dir), str(repo_id), str(version_id), str(file_name)
+        )
         # Download Model File
         file_downloader(
             url=download_url,
@@ -911,7 +948,9 @@ def search_civitai(search_word: str, **kwargs) -> Union[str, SearchResult, None]
             model_path=model_path,
             loading_method=output_info["loading_method"],
             checkpoint_format=output_info["checkpoint_format"],
-            repo_status=RepoStatus(repo_id=repo_name, repo_hash=repo_id, version=version_id),
+            repo_status=RepoStatus(
+                repo_id=repo_name, repo_hash=repo_id, version=version_id
+            ),
             model_status=ModelStatus(
                 search_word=search_word,
                 site_url=f"https://civitai.com/models/{repo_id}?modelVersionId={version_id}",
@@ -1045,7 +1084,9 @@ class AutoConfig:
                     f"Token {token} already in tokenizer vocabulary. Please choose a different token name or remove {token} and embedding from the tokenizer and text encoder."
                 )
 
-        expected_shape = text_encoder.get_input_embeddings().weight.shape[-1]  # Expected shape of tokenizer
+        expected_shape = text_encoder.get_input_embeddings().weight.shape[
+            -1
+        ]  # Expected shape of tokenizer
 
         for search_word in pretrained_model_name_or_paths:
             if isinstance(search_word, str):
@@ -1076,16 +1117,23 @@ class AutoConfig:
                     f"textual_inversion_path: {search_word} -> {textual_inversion_path.model_status.site_url}"
                 )
 
-                pretrained_model_name_or_paths[pretrained_model_name_or_paths.index(search_word)] = (
-                    textual_inversion_path.model_path
-                )
+                pretrained_model_name_or_paths[
+                    pretrained_model_name_or_paths.index(search_word)
+                ] = textual_inversion_path.model_path
 
         self.load_textual_inversion(
-            pretrained_model_name_or_paths, token=tokens, tokenizer=tokenizer, text_encoder=text_encoder, **kwargs
+            pretrained_model_name_or_paths,
+            token=tokens,
+            tokenizer=tokenizer,
+            text_encoder=text_encoder,
+            **kwargs,
         )
 
     def auto_load_lora_weights(
-        self, pretrained_model_name_or_path_or_dict: Union[str, Dict[str, torch.Tensor]], adapter_name=None, **kwargs
+        self,
+        pretrained_model_name_or_path_or_dict: Union[str, Dict[str, torch.Tensor]],
+        adapter_name=None,
+        **kwargs,
     ):
         r"""
         Load LoRA weights specified in `pretrained_model_name_or_path_or_dict` into `self.unet` and
@@ -1129,10 +1177,12 @@ class AutoConfig:
             logger.warning(f"trained_words: {lora_path.extra_status.trained_words}")
             pretrained_model_name_or_path_or_dict = lora_path.model_path
 
-        self.load_lora_weights(pretrained_model_name_or_path_or_dict, adapter_name=adapter_name, **kwargs)
+        self.load_lora_weights(
+            pretrained_model_name_or_path_or_dict, adapter_name=adapter_name, **kwargs
+        )
 
 
-class EasyPipelineForText2Image(AutoPipelineForText2Image, AutoConfig):
+class EasyPipelineForText2Image(AutoPipelineForText2Image):
     r"""
     [`EasyPipelineForText2Image`] is a generic pipeline class that instantiates a text-to-image pipeline class. The
     specific underlying pipeline class is automatically selected from either the
@@ -1267,20 +1317,25 @@ class EasyPipelineForText2Image(AutoPipelineForText2Image, AutoConfig):
         kwargs.update(_status)
 
         # Search for the model on Hugging Face and get the model status
-        hf_checkpoint_status = search_huggingface(pretrained_model_link_or_path, **kwargs)
-        logger.warning(f"checkpoint_path: {hf_checkpoint_status.model_status.download_url}")
+        hf_checkpoint_status = search_huggingface(
+            pretrained_model_link_or_path, **kwargs
+        )
+        logger.warning(
+            f"checkpoint_path: {hf_checkpoint_status.model_status.download_url}"
+        )
         checkpoint_path = hf_checkpoint_status.model_path
 
         # Check the format of the model checkpoint
         if hf_checkpoint_status.loading_method == "from_single_file":
             # Load the pipeline from a single file checkpoint
-            return load_pipeline_from_single_file(
+            pipeline = load_pipeline_from_single_file(
                 pretrained_model_or_path=checkpoint_path,
                 pipeline_mapping=SINGLE_FILE_CHECKPOINT_TEXT2IMAGE_PIPELINE_MAPPING,
                 **kwargs,
             )
         else:
-            return cls.from_pretrained(checkpoint_path, **kwargs)
+            pipeline = cls.from_pretrained(checkpoint_path, **kwargs)
+        return add_methods(pipeline)
 
     @classmethod
     def from_civitai(cls, pretrained_model_link_or_path, **kwargs):
@@ -1524,8 +1579,12 @@ class EasyPipelineForImage2Image(AutoPipelineForImage2Image):
         kwargs.update(_parmas)
 
         # Search for the model on Hugging Face and get the model status
-        hf_checkpoint_status = search_huggingface(pretrained_model_link_or_path, **kwargs)
-        logger.warning(f"checkpoint_path: {hf_checkpoint_status.model_status.download_url}")
+        hf_checkpoint_status = search_huggingface(
+            pretrained_model_link_or_path, **kwargs
+        )
+        logger.warning(
+            f"checkpoint_path: {hf_checkpoint_status.model_status.download_url}"
+        )
         checkpoint_path = hf_checkpoint_status.model_path
 
         # Check the format of the model checkpoint
@@ -1639,11 +1698,12 @@ class EasyPipelineForImage2Image(AutoPipelineForImage2Image):
         checkpoint_path = checkpoint_status.model_path
 
         # Load the pipeline from a single file checkpoint
-        return load_pipeline_from_single_file(
+        pipeline = load_pipeline_from_single_file(
             pretrained_model_or_path=checkpoint_path,
             pipeline_mapping=SINGLE_FILE_CHECKPOINT_IMAGE2IMAGE_PIPELINE_MAPPING,
             **kwargs,
         )
+        return add_methods(pipeline)
 
 
 class EasyPipelineForInpainting(AutoPipelineForInpainting):
@@ -1782,20 +1842,25 @@ class EasyPipelineForInpainting(AutoPipelineForInpainting):
         kwargs.update(_status)
 
         # Search for the model on Hugging Face and get the model status
-        hf_checkpoint_status = search_huggingface(pretrained_model_link_or_path, **kwargs)
-        logger.warning(f"checkpoint_path: {hf_checkpoint_status.model_status.download_url}")
+        hf_checkpoint_status = search_huggingface(
+            pretrained_model_link_or_path, **kwargs
+        )
+        logger.warning(
+            f"checkpoint_path: {hf_checkpoint_status.model_status.download_url}"
+        )
         checkpoint_path = hf_checkpoint_status.model_path
 
         # Check the format of the model checkpoint
         if hf_checkpoint_status.loading_method == "from_single_file":
             # Load the pipeline from a single file checkpoint
-            return load_pipeline_from_single_file(
+            pipeline = load_pipeline_from_single_file(
                 pretrained_model_or_path=checkpoint_path,
                 pipeline_mapping=SINGLE_FILE_CHECKPOINT_INPAINT_PIPELINE_MAPPING,
                 **kwargs,
             )
         else:
-            return cls.from_pretrained(checkpoint_path, **kwargs)
+            pipeline = cls.from_pretrained(checkpoint_path, **kwargs)
+        return add_methods(pipeline)
 
     @classmethod
     def from_civitai(cls, pretrained_model_link_or_path, **kwargs):
@@ -1895,8 +1960,9 @@ class EasyPipelineForInpainting(AutoPipelineForInpainting):
         checkpoint_path = checkpoint_status.model_path
 
         # Load the pipeline from a single file checkpoint
-        return load_pipeline_from_single_file(
+        pipeline = load_pipeline_from_single_file(
             pretrained_model_or_path=checkpoint_path,
             pipeline_mapping=SINGLE_FILE_CHECKPOINT_INPAINT_PIPELINE_MAPPING,
             **kwargs,
         )
+        return add_methods(pipeline)
